#!/usr/bin/python3
import sys
import os
import json
import datetime as dt
import requests

def get_config():
    builtin_cfg =     {
     'SCLAB_API_URL': 'https://umwelttechnik-swc.uibk.ac.at/api',
     'SCLAB_API_USER': '',
     'SCLAB_API_PASS': '',
    }

    env_cfg = {}
    for key in builtin_cfg:
        env_cfg[key] = os.getenv(key)

    if None in env_cfg.values():
        log('INFO', 'no configuration environment found, using built-in development credentials')
        return builtin_cfg

    return env_cfg

def log(msg_type, msg):
    now = dt.datetime.now()
    print('{}: {}: {}'.format(now, msg_type, msg), file=sys.stderr)

def read_datapoints(cfg, sensor_id, begin, end, sampling):
    url = '{}/datapoint/{}/read'.format(cfg['SCLAB_API_URL'], sensor_id)
    result = requests.get(url, params={'begin': begin, 'end': end, 'sampling': sampling}, auth=(cfg['SCLAB_API_USER'], cfg['SCLAB_API_PASS']))
    if result.status_code >= 400:
        return False, 'error: {}'.format(result.status_code)

    return True, result.json()

def read_last_value(cfg, sensor_id):
    url = '{}/sensor/{}/info'.format(cfg['SCLAB_API_URL'], sensor_id)
    result = requests.get(url, auth=(cfg['SCLAB_API_USER'], cfg['SCLAB_API_PASS']))
    if result.status_code >= 400:
        return False, 'error: {}'.format(result.status_code)

    return True, result.json()    

def print_usage_str_and_exit(progname, code):
    progname_base = os.path.basename(progname)
    out = '''\
{} <cmd> [<arg0> ... <argN>]

{} status                                show service status
{} read <id> <from> <to> [<sampling>]    read (optionally sampled) datapoints of <sensorid>
{} write <id> <data>                     insert datapoints <data> for <sensorid>
{} sensors                               show all available sensors
{} help                                  this information

Use \'{} help <cmd>\' to get more information about a specific command.\
    '''
    nprogname = out.count('{}')
    target = sys.stdout if code == 0 else sys.stderr
    print(out.format(*(nprogname * [progname_base])), file=target)
    sys.exit(code)

if __name__ == '__main__':
    progname = sys.argv[0]
    args = sys.argv[1:]

    if len(args) == 0:
        print_usage_str_and_exit(progname, 1)
    
    nargs = len(args)
    cmd = args[0]

    cfg = get_config()

    if nargs >= 4 and cmd == 'read':
        sampling = 'best'
        sensor_id = args[1]
        from_str = args[2]
        to_str = args[3]
        if nargs > 4:
            sampling = args[4]

        success, val = read_datapoints(cfg, sensor_id, from_str, to_str, sampling)
        if not success:
            err_msg = val
            print('fatal: {}'.format(err_msg), file=sys.stderr)
            sys.exit(1)

        print(json.dumps(val))

    elif nargs == 1 and cmd == 'help':
        print_usage_str_and_exit(progname, 0)

    else:
        print_usage_str_and_exit(progname, 1)

